@@ -612,8 +612,8 @@ namespace mfront {
          << "return false;\n"
          << "}\n";
       auto cr = SupportedTypes::TypeSize{};  // current row
-      auto cc = SupportedTypes::TypeSize{};  // current column
       for (const auto& iv1 : isvs) {
+        auto cc = SupportedTypes::TypeSize{};  // current column
         for (const auto& iv2 : isvs) {
           if (std::find_if(invert_jacobian_blocks.begin(),
                            invert_jacobian_blocks.end(),
@@ -633,9 +633,10 @@ namespace mfront {
           derivative_view.derivative_row_position = cr;
           derivative_view.derivative_column_position = cc;
           this->writeDerivativeView(os, derivative_view);
-          // update the row position
+          // update the column position
           cc += mfront::getTypeSize(iv2);
         }
+        // update the row position
         cr += mfront::getTypeSize(iv1);
       }
     }
@@ -900,7 +901,7 @@ namespace mfront {
     const auto v2 = VariableDescription{"StrainStensor", "\u03B5\u1D57\u1D52",
                                         "eto", 1u, 0u};
     for (std::size_t i = 0; i != isvs.size(); ++i) {
-      os << "TFEL_HOST_DEVICE [[nodiscard]] bool\ncomputePartialJacobianInvert("
+      os << "[[nodiscard]] TFEL_HOST_DEVICE bool\ncomputePartialJacobianInvert("
             "const tfel::math::TinyPermutation<"
          << n << ">& jacobian_permutation, ";
       for (std::size_t i2 = 0; i2 <= i;) {
@@ -1039,7 +1040,7 @@ namespace mfront {
     os << "/*!\n"
        << " * \\brief Integrate behaviour law over the time step\n"
        << " */\n"
-       << "TFEL_HOST_DEVICE [[nodiscard]] IntegrationResult ";
+       << "[[nodiscard]] TFEL_HOST_DEVICE IntegrationResult ";
     if (this->bd.hasAttribute(h, BehaviourData::hasConsistentTangentOperator)) {
       os << "integrate(const SMFlag smflag,const SMType smt) override{\n";
     } else {
